Package: spoon
Title: Address the Mean-variance Relationship in Spatial
        Transcriptomics Data
Version: 1.9.0
Authors@R: c(
    person(given = "Kinnary",
           family = "Shah",
           role = c("aut", "cre"),
           email = "kinnaryshahh@gmail.com",
           comment = c(ORCID = "0000-0001-7098-2116")),
    person("Boyi", "Guo",
           email = "boyi.guo.work@gmail.com",
           role = c("aut"),
           comment = c(ORCID = "0000-0003-2950-2349")),
    person("Stephanie C.", "Hicks", 
           email = "shicks19@jhu.edu", 
           role = c("aut"), 
           comment = c(ORCID = "0000-0002-7858-0231")))
Description: This package addresses the mean-variance relationship in spatially
  resolved transcriptomics data. Precision weights are generated for individual
  observations using Empirical Bayes techniques. These weights are used to
  rescale the data and covariates, which are then used as input in spatially
  variable gene detection tools.
URL: https://github.com/kinnaryshah/spoon
BugReports: https://github.com/kinnaryshah/spoon/issues
Imports: SpatialExperiment, BRISC, nnSVG, BiocParallel, Matrix,
        methods, SummarizedExperiment, stats, utils, scuttle
License: MIT + file LICENSE
Encoding: UTF-8
biocViews: Spatial, SingleCell, Transcriptomics, GeneExpression,
        Preprocessing
Depends: R (>= 4.4)
Roxygen: list(markdown = TRUE)
Suggests: testthat, STexampleData, knitr, rmarkdown, BiocStyle
Config/testthat/edition: 3
VignetteBuilder: knitr
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/spoon
git_branch: devel
git_last_commit: a0a9ed9
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 01:23:44 UTC; biocbuild
Author: Kinnary Shah [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7098-2116>),
  Boyi Guo [aut] (ORCID: <https://orcid.org/0000-0003-2950-2349>),
  Stephanie C. Hicks [aut] (ORCID:
    <https://orcid.org/0000-0002-7858-0231>)
Maintainer: Kinnary Shah <kinnaryshahh@gmail.com>
