# Generated by roxygen2: do not edit by hand

export(generate_weights)
export(weighted_nnSVG)
importFrom(BRISC,BRISC_estimation)
importFrom(BRISC,BRISC_neighbor)
importFrom(BRISC,BRISC_order)
importFrom(BRISC,BRISC_prediction)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bplapply)
importFrom(Matrix,rowMeans)
importFrom(SpatialExperiment,spatialCoords)
importFrom(SummarizedExperiment,`assay<-`)
importFrom(SummarizedExperiment,`rowData<-`)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,rowData)
importFrom(methods,is)
importFrom(nnSVG,filter_genes)
importFrom(nnSVG,nnSVG)
importFrom(scuttle,computeLibraryFactors)
importFrom(scuttle,logNormCounts)
importFrom(stats,predict)
importFrom(stats,sd)
importFrom(stats,smooth.spline)
importFrom(utils,packageVersion)
